<?php
/**
 * Popup Box Module Pro
 * 
 * @author  Cuispi
 * @version 2.0.3
 * @license Commercial License
 * @package system
 * @subpackage  system.library.popup_box
 */
class PopupBoxBase {

  /**
   * Development Mode
   *
   * @var boolean True or false
   */
  private $dev = false;
  
  /**
   * Code name of this module.
   *
   * @var string
   */   
  protected $code = 'popup_box';
  
  /**
   * Holds the current version of the module.
   *
   * @var string
   */
  protected $version = '2.0.3';  

  /**
   * The partial path to the module directory.
   *
   * @var string
   */  
  protected $module_path;

  /**
   * The instantiated model class name of this extension.
   *
   * @var string
   */  
  protected $model_name;  
  
  /**
   * Hold the default global parameters
   *
   * @var mixed
   */
  protected $global_params = array();
  
  /**
   * Hold the skin data.
   *
   * @var array
   */  
  protected $skin = array();
  
  /**
   * Variable for the Logger class instance.
   *
   * @var Logger instance
   */
  protected $logger;
  
  /**
   * The singleton instance of the PopupBoxBase class.
   *
   * @var PopupBoxBase
   */
  protected static $instance;  	
	
  /**
   * Create a new PopupBoxBase instance.
   *
   */  
  private function __construct() {}  

  /**
   * Get or create the singleton PopupBoxBase instance.
   *
   * @return PopupBoxBase
   */  
  public static function getInstance() {
    if (!(self::$instance instanceof self)) {
      self::$instance = new self();
    }

    return self::$instance;        
  }

  /**
   * Initialize and setup the PopupBoxBase instance.
   *
   * @param  object $registry 
   * @return $this
   */ 
  public function init($registry) {
		$this->load = $registry->get('load');
		$this->language = $registry->get('language');
		$this->document = $registry->get('document');
		$this->customer = $registry->get('customer');
		$this->request = $registry->get('request');
		$this->config = $registry->get('config');
    
		$this->registry = $registry;
    
    if (version_compare(VERSION, '2.3.0.0', '<')) { // for OpenCart 2.2.0.0 or earlier.
      $this->module_path = 'module/' . $this->code;
      $this->model_name = 'model_module_' . $this->code;
    } else {
      $this->module_path = 'extension/module/' . $this->code;
      $this->model_name = 'model_extension_module_' . $this->code;
    }    
    
    // Logger
    $this->logger = new Log($this->code . '.log');    
    
    return $this;
  } 
  
  /**
   * Show the module for a given module id.
   * Called statically with HTML::show()
   *
   * @param  integer $id  Module id (OpenCart 2.0.1.0 or later)
   * @return $this->render()
   */   
  public function show($id) {
    if (!$this->config->get($this->code . '_status')) {
      return false;
    }
    
    $module_id = (int)$id;
    
    $this->load->model('extension/module');
    $setting = $this->registry->get('model_extension_module')->getModule($module_id);
    
    $setting['module_id'] = $module_id;    

    return $this->render($setting);
  }
  
  /**
   * Render HTML content
   *
   * @param  array $setting
   * @return view
   */ 
  public function render($setting) {
    // Skip executing code when the variable $setting is an empty array. 
    if (!isset($setting['status'])) {
      return false;
    }
    
    if (!$this->config->get($this->code . '_status')) {
      return false;
    }
    
    static $module = 0;

    
    $this->language->load($this->module_path);

    $this->load->model($this->module_path);
    
    //
    // Skin
    // --------------------------------------------------       
    
    if ($setting['skin']) {
      $this->skin = array(
          'key' => $setting['skin'],
          'template' => $setting['skin_template']
      );

    } else {
      $this->skin = array(
          'key' => false,
          'template' => false
      );
    }
    
    //
    // Stores
    // --------------------------------------------------    

    $stores = array();
    
    if (isset($setting['stores'])) {
      $stores = (array)$setting['stores'];
    }

    if (!$stores || !in_array($this->config->get('config_store_id'), $stores)) {
      return false;
    }

    //
    // Customer groups
    // --------------------------------------------------
    
    $customer_group_display = array();
    
    if (isset($setting['customer_groups'])) {
      $customer_group_display = (array)$setting['customer_groups'];
    }

    if (!$this->customer->isLogged() && !in_array(0, $customer_group_display)) {
      return false;
    }
    
    $customer_group_id = null;
    
    if ($this->customer->isLogged()) {
      $this->load->model('account/customer_group');
      $customer_group_info = $this->registry->get('model_account_customer_group')->getCustomerGroup($this->config->get('config_customer_group_id'));

      if (!in_array($customer_group_info['customer_group_id'], $customer_group_display)) {
        return false;
      }
      
      $customer_group_id = $customer_group_info['customer_group_id'];
    }

    //
    // Categories
    // --------------------------------------------------
    
    if (isset($setting['categories']) && $setting['categories']) {

      if (isset($this->request->get['route'])) {
        $route = $this->request->get['route'];
      } else {
        $route = 'common/home';
      }

      if ($route == 'product/category' && isset($this->request->get['path'])) { // Executed only on Category pages.
        // Extract category ids from category paths
        $paths = explode('_', (string)$this->request->get['path']);

        if ($setting['on_selected_categories'] == 0) { // Hide on selected pages
          if (in_array(end($paths), $setting['categories'])) {
            return false;
          }

        } else if ($setting['on_selected_categories'] == 1) { // Show on selected pages
          if (!in_array(end($paths), $setting['categories'])) {
            return false;
          }
        }
      }
    }
    
    //
    // Login status
    // --------------------------------------------------
    
    if (isset($setting['login_status']) && $setting['login_status']) {
      
      if ($setting['login_status'] == 'show') {
        if (!$this->customer->isLogged()) {
          return false;
        }
        
      } elseif ($setting['login_status'] == 'hide') {
        if ($this->customer->isLogged()) {
          return false;
        }
      }
    }
    
    //
    // Start date and end date
    // --------------------------------------------------
    
    $defaultTimezone = date_default_timezone_get();
    
    if (isset($setting['timezone']) && $setting['timezone']) {
      date_default_timezone_set($setting['timezone']);
    }

    $now = new DateTime('now');

    if (isset($setting['date_start']) && $setting['date_start']) {
      $startDate = new DateTime($setting['date_start']);

      if ($now < $startDate) {
        return;
      }
    }

    if (isset($setting['date_end']) && $setting['date_end']) {
      $endDate = new DateTime($setting['date_end']);

      if ($now >= $endDate) {
        return;
      }    
    }

    if (isset($setting['timezone']) && $setting['timezone']) {
      date_default_timezone_set($defaultTimezone);
    }
    
    unset($defaultTimezone, $now, $startDate, $endDate);
    
    //
    // Stylesheet
    // --------------------------------------------------  
    
    if ((bool)$setting['use_default_stylesheet']) {
      if (is_file('catalog/view/theme/' . $this->config->get('config_template') . '/stylesheet/' . $this->code . '/' . $this->code . '.css')) {
        $this->document->addStyle('catalog/view/theme/' . $this->config->get('config_template') . '/stylesheet/' . $this->code . '/' . $this->code . '.css' . ((bool)$this->dev ? '?'.  time() : ''), 'stylesheet', 'all');
      } else {
        $this->document->addStyle('catalog/view/theme/default/stylesheet/' . $this->code . '/' . $this->code . '.css' . ((bool)$this->dev ? '?'.  time() : ''), 'stylesheet', 'all');
      }
    }
    
    // Skin
    if ($this->skin['key']) {
      if (is_file('catalog/view/theme/' . $this->config->get('config_template') . '/stylesheet/' . $this->code . '/skins/' . $this->skin['key'] . '.css')) {
        $this->document->addStyle('catalog/view/theme/' . $this->config->get('config_template') . '/stylesheet/' . $this->code . '/skins/' . $this->skin['key'] . '.css' . ((bool)$this->dev ? '?'.  time() : ''), 'stylesheet', 'all');
      } elseif (is_file('catalog/view/theme/default/stylesheet/' . $this->code . '/skins/' . $this->skin['key'] . '.css')) {
        $this->document->addStyle('catalog/view/theme/default/stylesheet/' . $this->code . '/skins/' . $this->skin['key'] . '.css' . ((bool)$this->dev ? '?'.  time() : ''), 'stylesheet', 'all');
      }
    }
    
    // External CSS
    if ($setting['external_css']) {
      $external_css_paths = explode("\r\n", (string) $setting['external_css']);

      foreach ($external_css_paths as $css_file_path) {
        $css_file_path = trim($css_file_path);
        
        if (!$css_file_path) {
          continue;
        }
        
        // Replace [THEME] with the current template name.
        $path = preg_replace("/\[THEME\]/", $this->config->get('config_template'), $css_file_path);

        if (is_file($path)) {
          $this->document->addStyle($path);
        } else {
          // Replace [THEME] with the default template name.
          $this->document->addStyle(preg_replace("/\[THEME\]/", 'default', $css_file_path));
        }
      }
    }
    
    unset($path);

    //
    // Javascript
    // --------------------------------------------------  

    if ((bool)$setting['use_default_javascript']) {
      if (is_file('catalog/view/javascript/' . $this->code . '/' . $this->code . '_remodal' . ((bool)$this->dev ? '' : '.min') . '.js')) {
        $this->document->addScript('catalog/view/javascript/' . $this->code . '/' . $this->code . '_remodal' . ((bool)$this->dev ? '' : '.min') . '.js' . ((bool)$this->dev ? '?'.  time() : ''));
      }

      if (is_file('catalog/view/javascript/' . $this->code . '/' . $this->code . ((bool)$this->dev ? '' : '.min') . '.js')) {
        $this->document->addScript('catalog/view/javascript/' . $this->code . '/' . $this->code . ((bool)$this->dev ? '' : '.min') . '.js' . ((bool)$this->dev ? '?'.  time() : ''));
      }
    }
    
    // External JS
    if ($setting['external_js']) {
      $external_js_paths = explode("\r\n", (string) $setting['external_js']);

      foreach ($external_js_paths as $js_file_path) {
        $js_file_path = trim($js_file_path);
        
        if (!$js_file_path) {
          continue;
        }
        
        // Replace [THEME] with the current template name.
        $path = preg_replace("/\[THEME\]/", $this->config->get('config_template'), $js_file_path);

        if (is_file($path)) {
          $this->document->addScript($path);
        } else {
          // Replace [THEME] with the default template name.
          $this->document->addScript(preg_replace("/\[THEME\]/", 'default', $js_file_path));
        }        
        
      }
    }

    //
    // Main Content
    // --------------------------------------------------   
    
    $data = array();
    
    // Text  
    $data['text_remodal_close'] = $this->language->get('text_remodal_close');
    $data['text_dont_show_me_again'] = $this->language->get('text_dont_show_me_again');
    
    // Button
    $data['button_remodal_cancel'] = $this->language->get('button_remodal_cancel');
    $data['button_remodal_confirm'] = $this->language->get('button_remodal_confirm');
    
    // Error
    $data['error_html_form_not_enabled'] = $this->language->get('error_html_form_not_enabled');
    
    
    // Show title
    if ((bool)$setting['show_title']) {
      $data['show_title'] = (bool)$setting['show_title'];
    } else {
      $data['show_title'] = false;
    }
    
    // Heading Title
    if (isset($setting['title'][(int) $this->config->get('config_language_id')]) && $setting['title'][(int) $this->config->get('config_language_id')]) {
      $data['heading_title'] = html_entity_decode($setting['title'][(int) $this->config->get('config_language_id')]);
    } else {
      $data['heading_title'] = false;
    }

    // Title link
    if ($setting['title_link']) {
      $data['heading_title_link'] = $setting['title_link'];
    } else {
      $data['heading_title_link'] = "";
    }
    
    // Use HTML form 
    if ((bool)$setting['use_form']) {
      $data['use_form'] = (bool)$setting['use_form'];
    } else {
      $data['use_form'] = false;
    }   
    
    // Form action
    if ($setting['form_action']) {
      $data['form_action'] = $setting['form_action'];
    } else {
      $data['form_action'] = "";
    }
    
    // Form method
    if ($setting['form_method']) {
      $data['form_method'] = $setting['form_method'];
    } else {
      $data['form_method'] = "";
    }
    
    // Html content
    if (isset($setting['html_content'][(int) $this->config->get('config_language_id')]) && $setting['html_content'][(int) $this->config->get('config_language_id')]) {
      $data['html_content'] = html_entity_decode($setting['html_content'][(int) $this->config->get('config_language_id')]);
    } else {
      $data['html_content'] = "";
    }
    
    // Show a "Don't show me this message again" link
    if ((bool)$setting['show_dont_show_me']) {
      $data['show_dont_show_me'] = (bool)$setting['show_dont_show_me'];
    } else {
      $data['show_dont_show_me'] = false;
    }

    // "Don't show me this message again" text
    if (isset($setting['dont_show_me_text'][(int) $this->config->get('config_language_id')]) && $setting['dont_show_me_text'][(int) $this->config->get('config_language_id')]) {
      $data['dont_show_me_text'] = html_entity_decode($setting['dont_show_me_text'][(int) $this->config->get('config_language_id')]);
    } else {
      $data['dont_show_me_text'] = false;
    }
    
    // Show OK button
    if ((bool)$setting['show_confirm_btn']) {
      $data['show_confirm_btn'] = (bool)$setting['show_confirm_btn'];
    } else {
      $data['show_confirm_btn'] = false;
    }       

    // OK button text
    if (isset($setting['confirm_btn_text'][(int) $this->config->get('config_language_id')]) && $setting['confirm_btn_text'][(int) $this->config->get('config_language_id')]) {
      $data['confirm_btn_text'] = html_entity_decode($setting['confirm_btn_text'][(int) $this->config->get('config_language_id')]);
    } else {
      $data['confirm_btn_text'] = false;
    }    

    // Show Cancel button
    if ((bool)$setting['show_cancel_btn']) {
      $data['show_cancel_btn'] = (bool)$setting['show_cancel_btn'];
    } else {
      $data['show_cancel_btn'] = false;
    }       

    // Cancel button text
    if (isset($setting['cancel_btn_text'][(int) $this->config->get('config_language_id')]) && $setting['cancel_btn_text'][(int) $this->config->get('config_language_id')]) {
      $data['cancel_btn_text'] = html_entity_decode($setting['cancel_btn_text'][(int) $this->config->get('config_language_id')]);
    } else {
      $data['cancel_btn_text'] = false;
    } 

    // Show Close (X) button
    if ((bool)$setting['show_close_btn']) {
      $data['show_close_btn'] = (bool)$setting['show_close_btn'];
    } else {
      $data['show_close_btn'] = false;
    }     
    
    // Popup Max Width
    if ($setting['modal_max_width']) {
      $data['modal_max_width'] = $setting['modal_max_width'];
    } else {
      $data['modal_max_width'] = false;
    }

    // Popup Max Height
    if ($setting['modal_max_height']) {
      $data['modal_max_height'] = $setting['modal_max_height'];
    } else {
      $data['modal_max_height'] = false;
    }

    // Animation Effect
    if ($setting['anim_effect']) {
      $data['anim_effect'] = $setting['anim_effect'];
    } else {
      $data['anim_effect'] = false;
    }

    // Skin
    $data['skin_key'] = $this->skin['key'];
    $data['skin_template'] = $this->skin['template'];
    
    // Use a custom background color and opacity 
    if ((bool)$setting['use_overlay_color']) {
      $data['use_overlay_color'] = (bool)$setting['use_overlay_color'];
    } else {
      $data['use_overlay_color'] = false;
    }    

    // Overlay Color
    if ($setting['overlay_color']) {
      $data['overlay_color'] = $setting['overlay_color'];
    } else {
      $data['overlay_color'] = false;
    }

    // Overlay Opacity
    if ($setting['overlay_opacity']) {
      $data['overlay_opacity'] = $setting['overlay_opacity'];
    } else {
      $data['overlay_opacity'] = false;
    }
    
    // Overlay in RGBA colors
    if ($setting['overlay_color']) {
      $overlayRgb = $this->hex2Rgb($setting['overlay_color']);
      
      if (filter_var($setting['overlay_opacity'], FILTER_VALIDATE_INT, array('options' => array('min_range' => 0, 'max_range' => 100)))) {
        $overlayRgb[] = $setting['overlay_opacity'] / 100;
      }
      $data['overlay_rgba'] = implode(', ', $overlayRgb);
    } else {
      $data['overlay_rgba'] = false;
    }

    // Add an additional background layer under the overlay 
    if ((bool)$setting['add_bg_layer']) {
      $data['add_bg_layer'] = (bool)$setting['add_bg_layer'];
    } else {
      $data['add_bg_layer'] = false;
    }

    // Background Filter
    if ($setting['bg_filter']) {
      $data['bg_filter'] = $setting['bg_filter'];
    } else {
      $data['bg_filter'] = false;
    }

    // Display
    if ($setting['display']) {
      $data['display'] = $setting['display'];
    } else {
      $data['display'] = false;
    }

    // Display Delay
    if ($setting['display_delay']) {
      $data['display_delay'] = (int)$setting['display_delay'];
    } else {
      $data['display_delay'] = false;
    }

    // Delay Type
    if ($setting['delay_type']) {
      $data['delay_type'] = $setting['delay_type'];
    } else {
      $data['delay_type'] = false;
    }

    // Display Scroll
    if ($setting['display_scroll']) {
      $data['display_scroll'] = (int)$setting['display_scroll'];
    } else {
      $data['display_scroll'] = false;
    }
    
    // Scroll Type
    if ($setting['scroll_type']) {
      $data['scroll_type'] = $setting['scroll_type'];
    } else {
      $data['scroll_type'] = false;
    }
    
    // Display Click
    if ($setting['display_click']) {
      $data['display_click'] = htmlspecialchars_decode($setting['display_click']);
    } else {
      $data['display_click'] = false;
    }

    // Make the Close (x) button act as the "Don't show me this message again" link
    if ((bool)$setting['dont_show_me_on_close']) {
      $data['dont_show_me_on_close'] = (bool)$setting['dont_show_me_on_close'];
    } else {
      $data['dont_show_me_on_close'] = false;
    }
    
    // Make the ESC key act as the "Don't show me this message again" link
    if ((bool)$setting['dont_show_me_on_escape']) {
      $data['dont_show_me_on_escape'] = (bool)$setting['dont_show_me_on_escape'];
    } else {
      $data['dont_show_me_on_escape'] = false;
    }
    
    // Make the Cancel button act as the "Don't show me this message again" link
    if ((bool)$setting['dont_show_me_on_cancel']) {
      $data['dont_show_me_on_cancel'] = (bool)$setting['dont_show_me_on_cancel'];
    } else {
      $data['dont_show_me_on_cancel'] = false;
    }
    
    // Make the OK button act as the "Don't show me this message again" link
    if ((bool)$setting['dont_show_me_on_confirm']) {
      $data['dont_show_me_on_confirm'] = (bool)$setting['dont_show_me_on_confirm'];
    } else {
      $data['dont_show_me_on_confirm'] = false;
    }
    
    // Make outside clicks act as the "Don't show me this message again" link
    if ((bool)$setting['dont_show_me_on_outside_click']) {
      $data['dont_show_me_on_outside_click'] = (bool)$setting['dont_show_me_on_outside_click'];
    } else {
      $data['dont_show_me_on_outside_click'] = false;
    }
    
    // Expiration time in days for the "Don't show me this message again" link
    if ($setting['expires_after']) {
      $data['expires_after'] = (int)$setting['expires_after'];
    } else {
      $data['expires_after'] = false;
    }
    
    // Close the pop-up when clicking outside of it (the overlay area)
    if ((bool)$setting['close_on_outside_click']) {
      $data['close_on_outside_click'] = (bool)$setting['close_on_outside_click'];
    } else {
      $data['close_on_outside_click'] = false;
    }    
    
    // Close the pop-up when pressing the ESC key 
    if ((bool)$setting['close_on_escape']) {
      $data['close_on_escape'] = (bool)$setting['close_on_escape'];
    } else {
      $data['close_on_escape'] = false;
    }    
    
    // Click event for OK button 
    if ($setting['on_confirm_click']) {
      $data['on_confirm_click'] = $setting['on_confirm_click'];
    } else {
      $data['on_confirm_click'] = false;
    }
    
    // Click event for OK button: Open link - Link URL
    if ($setting['on_confirm_click_ln_url']) {
      $data['on_confirm_click_ln_url'] = $setting['on_confirm_click_ln_url'];
    } else {
      $data['on_confirm_click_ln_url'] = '';
    }
    
    // Click event for OK button: Open link - Link target
    if ($setting['on_confirm_click_ln_trg']) {
      $data['on_confirm_click_ln_trg'] = $setting['on_confirm_click_ln_trg'];
    } else {
      $data['on_confirm_click_ln_trg'] = '';
    }
    
    // Click event for OK button: JavaScript code
    if ($setting['on_confirm_click_code']) {
      $data['on_confirm_click_code'] = htmlspecialchars_decode($setting['on_confirm_click_code']);
    } else {
      $data['on_confirm_click_code'] = '';
    }
    
    // Click event for Cancel button 
    if ($setting['on_cancel_click']) {
      $data['on_cancel_click'] = $setting['on_cancel_click'];
    } else {
      $data['on_cancel_click'] = false;
    }
    
    // Click event for Cancel button: Open link - Link URL
    if ($setting['on_cancel_click_ln_url']) {
      $data['on_cancel_click_ln_url'] = $setting['on_cancel_click_ln_url'];
    } else {
      $data['on_cancel_click_ln_url'] = '';
    }
    
    // Click event for Cancel button: Open link - Link target
    if ($setting['on_cancel_click_ln_trg']) {
      $data['on_cancel_click_ln_trg'] = $setting['on_cancel_click_ln_trg'];
    } else {
      $data['on_cancel_click_ln_trg'] = '';
    }
    
    // Click event for Cancel button: JavaScript code
    if ($setting['on_cancel_click_code']) {
      $data['on_cancel_click_code'] = htmlspecialchars_decode($setting['on_cancel_click_code']);
    } else {
      $data['on_cancel_click_code'] = '';
    }    
    
    // Determines whether or not to use the "before_initialize" callback.
    if ((bool)$setting['use_before_initialize']) {
      $data['use_before_initialize'] = (bool)$setting['use_before_initialize'];
    } else {
      $data['use_before_initialize'] = false;
    }    
    
    // A callback function that is to be called before the pop-up initializes 
    if ($setting['before_initialize_code']) {
      $data['before_initialize_code'] = htmlspecialchars_decode($setting['before_initialize_code'] . "\r\n");
    } else {
      $data['before_initialize_code'] = '';
    }
    
    // Determines whether or not to use the "after_initialize" callback.
    if ((bool)$setting['use_after_initialize']) {
      $data['use_after_initialize'] = (bool)$setting['use_after_initialize'];
    } else {
      $data['use_after_initialize'] = false;
    }    
    
    // A Callback function that is to be called after the pop-up initializes 
    if ($setting['after_initialize_code']) {
      $data['after_initialize_code'] = htmlspecialchars_decode($setting['after_initialize_code'] . "\r\n");
    } else {
      $data['after_initialize_code'] = '';
    }    
    
    // Determines whether or not to enable Hash Tracking to open a specific pop-up with a #hash in a URL.
    if ((bool)$setting['use_hash_tracking']) {
      $data['use_hash_tracking'] = (bool)$setting['use_hash_tracking'];
    } else {
      $data['use_hash_tracking'] = false;
    }
    
    // Hash
    if (isset($setting['hash']) && $setting['hash']) {
      $data['hash'] = $setting['hash']; 
    } else {
      $data['hash'] = false;
    }    
    
    // Tracking Code
    if (isset($setting['tracking_code']) && $setting['tracking_code']) {
      $data['tracking_code'] = $setting['tracking_code']; 
    } else {
      $data['tracking_code'] = false;
    }

    // Show ShareThis share buttons
    if ((bool)$setting['show_sharethis']) {
      $data['show_sharethis'] = (bool)$setting['show_sharethis'];
    } else {
      $data['show_sharethis'] = false;
    }
    
    // ShareThis Share Buttons
    if ($setting['sharethis_share_buttons']) {
      $sharethis_share_buttons = array();
      
      $array_sharethis_share_buttons = explode("\r\n", (string)$setting['sharethis_share_buttons']);

      foreach ($array_sharethis_share_buttons as $share_button) {
        if (strpos($share_button, ':') !== false) {
          // Split a string at the first occurrence of “:” (colon sign) into two $vars
          list($key, $value) = explode(':', $share_button, 2);

          if (trim($key)) {
            $sharethis_share_buttons[trim($key)] = html_entity_decode(trim($value));
          }
        }
      }
      
      $data['sharethis_share_buttons'] = $sharethis_share_buttons;
      
      unset($sharethis_share_buttons, $key, $value);
      
    } else {
      $data['sharethis_share_buttons'] = false;
    }    

    // ShareThis Publisher Key
    if ($setting['sharethis_pub_key']) {
      $data['sharethis_pub_key'] = $setting['sharethis_pub_key'];
    } else {
      $data['sharethis_pub_key'] = false;
    }
    
    // Current base URL for ShareThis
    $data['sharethis_base_url'] = $this->getCurrentUrl();
    
    //
    // Internal CSS
    // --------------------------------------------------     

    $internal_css = array();

    if (isset($setting['internal_css']) && trim($setting['internal_css']) == true) {
      $lines = explode("\r\n", (string) $setting['internal_css']);

      foreach ($lines as $line) {
        if (!$line) {
          continue;
        }
        
        $line = preg_replace("/\[modal\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module . '-remodal', $line);
        $line = preg_replace("/\[wrapper\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module . '-remodal-wrapper', $line);
        $line = preg_replace("/\[overlay\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module . '-remodal-overlay', $line);
        $line = preg_replace("/\[module\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module, $line);
        
        $internal_css[] = html_entity_decode($line);
      }
    }
    
    $data['internal_css'] = implode("\r\n", (array)$internal_css);         
    
    //
    // Internal JS
    // --------------------------------------------------     

    $internal_js = array();

    if (isset($setting['internal_js']) && trim($setting['internal_js']) == true) {
      $lines = explode("\r\n", (string) $setting['internal_js']);

      foreach ($lines as $line) {
        if (!$line) {
          continue;
        }
        
        $line = preg_replace("/\[modal\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module . '-remodal', $line);
        $line = preg_replace("/\[wrapper\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module . '-remodal-wrapper', $line);
        $line = preg_replace("/\[overlay\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module . '-remodal-overlay', $line);
        $line = preg_replace("/\[module\]/", '#' . str_replace('_', '-', $this->code) . '-' . $module, $line);    
        
        $internal_js[] = html_entity_decode($line);
      }
    }
    
    $data['internal_js'] = implode("\r\n", (array)$internal_js);
    
    //
    // Parameters
    // --------------------------------------------------      
    
    $this->global_params = array_merge($this->global_params, (array)$this->params($setting['params']));

    $data['params'] = $this->global_params;

    //
    // Misc
    // --------------------------------------------------       
    
    // Code
    $data['code'] = $this->code;
    
    // Module
    $data['module'] = $module++;
    $data['module_id'] = (int)$setting['module_id'];
    
    // JavaScript Debugging
    $config = $this->config->get($this->code . '_config');
    $data['js_debug'] = (bool)$config['js_debug'];

    //
    // Template
    // --------------------------------------------------      

    $template = null;
    
    if (version_compare(VERSION, '2.2.0.0', '<')) { // for OpenCart 2.1.0.2 or earlier.    
      if ($setting['use_custom_template']) {
        if (is_file(DIR_TEMPLATE . $this->config->get('config_template') . '/template/' . $this->module_path . '_' . $setting['template'] . '.tpl')) {
          $template = $this->config->get('config_template') . '/template/' . $this->module_path . '_' . $setting['template'] . '.tpl';
        } else {
          $template = 'default/template/module/' . $this->code . '_' . $setting['template'] . '.tpl';
        }
      } else {
        if (is_file(DIR_TEMPLATE . $this->config->get('config_template') . '/template/' . $this->module_path . '.tpl')) {
          $template = $this->config->get('config_template') . '/template/' . $this->module_path . '.tpl';
        } else {
          $template = 'default/template/module/' . $this->code . '.tpl';
        }
      }
    } else { // for OpenCart 2.2.0.0 or later.   
      if ($setting['use_custom_template']) {
        $template = $this->module_path . '_' . $setting['template'];
      } else {
        $template = $this->module_path;
      }
    }
    
    return $this->load->view($template, $data);  
  }
  
  /**
   * Get parameters from key:value string data where each element is separated by line feed.
   *
   * @param   string  $params
   * @return  array   A set of parameters
   */   
  protected function params($params) {
    if (empty($params)) {
      return null;
    }
    
    $result = array();

    $array_params = explode("\r\n", (string) $params);

    foreach ($array_params as $param) {
      if (strpos($param, ':') !== false) {
        // Split a string at the first occurrence of “:” (colon sign) into two $vars
        list($key, $value) = explode(':', $param, 2);

        if (trim($key)) {
          $value = html_entity_decode(trim($value));

          if (strtolower($value) === 'true') {
            $value = true;
          } elseif (strtolower($value) === 'false') {
            $value = false;
          } elseif (is_numeric($value) && strpos($value, '.') !== false) {
            $value = (float)$value;
          } elseif (is_numeric($value) && strpos($value, '.') !== true) {
            $value = (int)$value;
          }

          $result[trim($key)] = $value;
        }
      }
    }
    
    return $result;
  }
  
  /**
   * Convert HEX to RGB
   *
   * @param   string  $color
   * @return  array   A set of parameters
   */     
  protected function hex2Rgb($color) {
    if ( $color[0] == '#' ) {
      $color = substr( $color, 1 );
    }
    
    if ( strlen( $color ) == 6 ) {
      list( $r, $g, $b ) = array( $color[0] . $color[1], $color[2] . $color[3], $color[4] . $color[5] );
    } elseif ( strlen( $color ) == 3 ) {
      list( $r, $g, $b ) = array( $color[0] . $color[0], $color[1] . $color[1], $color[2] . $color[2] );
    } else {
      return false;
    }
    
    $r = hexdec($r);
    $g = hexdec($g);
    $b = hexdec($b);
    
    return array('red' => $r, 'green' => $g, 'blue' => $b);
  }
  
  /**
   * Get the Current Page URL
   *
   * @param void
   * @return string URL
   */  
  protected function getCurrentUrl() {
    $url = filter_input(INPUT_SERVER, 'HTTPS') == 'on' ? 'https://' : 'http://';
    
    $url .= filter_input(INPUT_SERVER, 'SERVER_PORT') != '80' 
              ? filter_input(INPUT_SERVER, 'SERVER_NAME') . ':' . filter_input(INPUT_SERVER, 'SERVER_PORT') . filter_input(INPUT_SERVER, 'REQUEST_URI') 
              : filter_input(INPUT_SERVER, 'SERVER_NAME') . filter_input(INPUT_SERVER, 'REQUEST_URI');
    
    return $url;
  }  
  
  /**
   * Get the current version of the module.
   *
   * @param void
   * @return string The current version of the module.
   */
  public function getVersion() {
    return $this->version;
  }
  
}
